@@ -33,6 +33,7 @@
 #include <glib.h>
 #include <libosso.h>
 #include <dbus/dbus.h>
+#include <libplayback/playback.h>
 #include <fcntl.h>
 #include <stdlib.h>
 
@@ -67,13 +68,29 @@
 #define HAL_MANAGER_SIGN "DeviceRemoved"
 
 /* A D-Bus rule for filtering events we're interested in */
-#define DBUS_RULE "type='signal',interface='" HAL_MANAGER_INTF \
-                  "',path='" HAL_MANAGER_PATH \
-                  "',member='" HAL_MANAGER_SIGN "'"
+#define DBUS_RULE_HAL "type='signal',interface='" HAL_MANAGER_INTF \
+                      "',path='" HAL_MANAGER_PATH \
+                      "',member='" HAL_MANAGER_SIGN "'"
 
 /* The name for headphones (w/ microphone?) on the D-Bus */
 #define HEADPHONE_UDI_NAME "/org/freedesktop/Hal/devices/computer_logicaldev_input_1"
 
+#define MCE_SIGNAL_INTF "com.nokia.mce.signal"
+#define MCE_SIGNAL_PATH "/com/nokia/mce/signal"
+#define MCE_CALL_STATE_SIG "sig_call_state_ind"
+
+#define MCE_CALL_ACTIVE "active"
+#define MCE_CALL_RINGING "ringing"
+#define MCE_CALL_DISCONNECT "none"
+
+/* A D-Bus rule for getting the call state (active/none/...) */
+#define DBUS_RULE_CALL "type='signal',interface='" MCE_SIGNAL_INTF \
+                       "',path='" MCE_SIGNAL_PATH \
+                       "',member='" MCE_CALL_STATE_SIG "'"
+
+/* Delay in milliseconds for the pause-after-call pause signal */
+#define POST_CALL_PAUSE_DELAY_MS 500
+
 /* Where the Media Player backend sits on the D-Bus */
 #ifdef DIABLO
 #    define MEDIA_SERVER_SRVC "com.nokia.osso_media_server"
@@ -98,11 +115,19 @@
 #define FMRADIO_PATH "/de/pycage/fmradio"
 #define FMRADIO_INTF "de.pycage.fmradio"
 
+/* Martin's MediaBox on the D-Bus */
+#define MEDIABOX_SRVC "de.pycage.mediabox"
+#define MEDIABOX_PATH "/de/pycage/mediabox/control"
+#define MEDIABOX_INTF "de.pycage.mediabox.control"
+
 typedef struct {
     DBusConnection* session_bus;
     DBusConnection* system_bus;
     osso_context_t* osso;
+    pb_playback_t *playback;
     gboolean initial;
+    gboolean call_active; /* True while a call is active */
+    gboolean unplugged_on_call; /* True when unplugged during active call */
 } Headphoned;
 
 /* This has to be globally accessible (for signal handling below) */
@@ -118,6 +143,25 @@ sig_handler(int sig G_GNUC_UNUSED)
     }
 }
 
+static void
+libplayback_state_request_handler(pb_playback_t   *pb,
+                                  enum pb_state_e req_state G_GNUC_UNUSED,
+                                  pb_req_t        *req,
+                                  void            *data G_GNUC_UNUSED)
+{
+    pb_playback_req_completed(pb, req);
+}
+
+static void
+libplayback_state_reply(pb_playback_t   *pb,
+                        enum pb_state_e  granted_state G_GNUC_UNUSED,
+                        const char      *reason G_GNUC_UNUSED,
+                        pb_req_t        *req,
+                        void            *data G_GNUC_UNUSED)
+{
+    pb_playback_req_completed(pb, req);
+}
+
 Headphoned*
 headphoned_new()
 {
@@ -129,7 +173,16 @@ headphoned_new()
     this->session_bus = (DBusConnection*)osso_get_dbus_connection(this->osso);
     this->system_bus = (DBusConnection*)osso_get_sys_dbus_connection(this->osso);
 
+    this->playback = pb_playback_new_2(this->session_bus,
+                                       PB_CLASS_MEDIA,
+                                       PB_FLAG_AUDIO,
+                                       PB_STATE_STOP,
+                                       (PBStateRequest)libplayback_state_request_handler,
+                                       NULL);
+
     this->initial = TRUE;
+    this->call_active = FALSE;
+    this->unplugged_on_call = FALSE;
 
     return this;
 }
@@ -148,6 +201,10 @@ broadcast_pause_signal(Headphoned* headphoned)
             "pause",
             NULL,
             DBUS_TYPE_INVALID);
+    pb_playback_req_state(headphoned->playback,
+                          PB_STATE_STOP,
+                          (PBStateReply)libplayback_state_reply,
+                          NULL);
 
     /* Panucci */
     if (dbus_bus_name_has_owner(headphoned->session_bus, PANUCCI_SRVC, NULL)) {
@@ -183,6 +240,27 @@ broadcast_pause_signal(Headphoned* headphoned)
                 NULL,
                 DBUS_TYPE_INVALID);
     }
+
+    /* MediaBox */
+    if (dbus_bus_name_has_owner(headphoned->session_bus, MEDIABOX_SRVC, NULL)) {
+        debug_msg("Sending pause signal to MediaBox");
+        osso_rpc_run(headphoned->osso,
+                MEDIABOX_SRVC,
+                MEDIABOX_PATH,
+                MEDIABOX_INTF,
+                "stop",
+                NULL,
+                DBUS_TYPE_INVALID);
+    }
+}
+
+/* A convenient GSourceFunc wrapper for broadcast_pause_signal */
+gboolean
+broadcast_pause_signal_later(gpointer data)
+{
+    Headphoned *headphoned = (Headphoned*)data;
+    broadcast_pause_signal(headphoned);
+    return FALSE;
 }
 
 /* Handler for messages from "wired" headphones (via sysfs) */
@@ -198,13 +276,22 @@ on_file_changed(GIOChannel* source, GIOCondition condition, gpointer data)
     g_io_channel_read_line(source, &result, NULL, NULL, NULL);
     g_strstrip(result);
 
-    if (g_ascii_strcasecmp(result, STATE_DISCONNECTED_STR) == 0) {
-        if (headphoned->initial == TRUE) {
-            debug_msg("Ignoring initial file change.");
-            headphoned->initial = FALSE;
-        } else {
+    if (headphoned->initial == TRUE) {
+        debug_msg("Ignoring initial file change.");
+        headphoned->initial = FALSE;
+    } else {
+        if (g_ascii_strcasecmp(result, STATE_DISCONNECTED_STR) == 0) {
             debug_msg("Broadcasting pause signal (cause: state file)");
             broadcast_pause_signal(headphoned);
+
+            if (headphoned->call_active == TRUE) {
+                /**
+                 * Remember this unplug event, so that we can
+                 * pause players after the call is finished.
+                 **/
+                debug_msg("Setting flag: unplugged_on_call");
+                headphoned->unplugged_on_call = TRUE;
+            }
         }
     }
 
@@ -223,13 +310,37 @@ on_msg_recieved(DBusConnection* connection G_GNUC_UNUSED, DBusMessage* message,
     dbus_message_iter_init(message, &iter);
     dbus_message_iter_get_basic(&iter, &result);
 
-    if (g_str_equal(result, HEADPHONE_UDI_NAME)) {
-        debug_msg("Broadcasting pause signal (cause: Hal via D-Bus)");
-        broadcast_pause_signal(headphoned);
-        return DBUS_HANDLER_RESULT_HANDLED;
-    } else {
-        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    if (g_str_equal(dbus_message_get_path(message), HAL_MANAGER_PATH)) {
+        if (g_str_equal(result, HEADPHONE_UDI_NAME)) {
+            debug_msg("Broadcasting pause signal (cause: Hal via D-Bus)");
+            broadcast_pause_signal(headphoned);
+            return DBUS_HANDLER_RESULT_HANDLED;
+        } else {
+            return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+        }
+    } else if (g_str_equal(dbus_message_get_path(message), MCE_SIGNAL_PATH)) {
+        if (g_str_equal(result, MCE_CALL_ACTIVE) ||
+                g_str_equal(result, MCE_CALL_RINGING)) {
+            debug_msg("Active call detected.");
+            headphoned->call_active = TRUE;
+            return DBUS_HANDLER_RESULT_HANDLED;
+        } else if (g_str_equal(result, MCE_CALL_DISCONNECT)) {
+            debug_msg("Call disconnect detected.");
+            headphoned->call_active = FALSE;
+            if (headphoned->unplugged_on_call == TRUE) {
+                headphoned->unplugged_on_call = FALSE;
+                debug_msg("Broadcasting pause signal (cause: call disconnect)");
+                g_timeout_add(POST_CALL_PAUSE_DELAY_MS,
+                        broadcast_pause_signal_later,
+                        (gpointer)headphoned);
+            }
+            return DBUS_HANDLER_RESULT_HANDLED;
+        } else {
+            return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+        }
     }
+
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
 int
@@ -253,8 +364,12 @@ main(int argc, char* argv[])
         warning_msg("Cannot open state file: %s", STATE_FILE);
     }
 
-    debug_msg("Registering D-Bus rule: %s", DBUS_RULE);
-    dbus_bus_add_match(headphoned->system_bus, DBUS_RULE, NULL);
+    debug_msg("Registering D-Bus rule: %s", DBUS_RULE_HAL);
+    dbus_bus_add_match(headphoned->system_bus, DBUS_RULE_HAL, NULL);
+
+    debug_msg("Registering D-Bus rule: %s", DBUS_RULE_CALL);
+    dbus_bus_add_match(headphoned->system_bus, DBUS_RULE_CALL, NULL);
+
     dbus_connection_add_filter(headphoned->system_bus, on_msg_recieved, headphoned, NULL);
 
     debug_msg("Entering GLib main loop...");
@@ -265,6 +380,7 @@ main(int argc, char* argv[])
         g_io_channel_unref(state);
     }
 
+    pb_playback_destroy(headphoned->playback);
     osso_deinitialize(headphoned->osso);
     g_free(headphoned);
 
